using System.Linq;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure;

namespace BankWebRole
{
  public class WebRole : RoleEntryPoint
  {
    public override bool OnStart()
    {
      // Get the factory configuration so that it can be edited
      DiagnosticMonitorConfiguration config = DiagnosticMonitor.GetDefaultInitialConfiguration();

      // Set scheduled transfer interval for infrastructure logs to 1 minute
      config.DiagnosticInfrastructureLogs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);

      // Specify a logging level to filter records to transfer
      config.DiagnosticInfrastructureLogs.ScheduledTransferLogLevelFilter = LogLevel.Error;

      // Set scheduled transfer interval for user's Windows Azure Logs to 1 minute
      config.Logs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);

      DiagnosticMonitor.Start("DiagnosticsConnectionString", config);

      // For information on handling configuration changes
      // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
      RoleEnvironment.Changing += RoleEnvironmentChanging;
      CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
      {
        configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));
        RoleEnvironment.Changed += (sender, arg) =>
        {
          if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
              .Any((change) => (change.ConfigurationSettingName == configName)))
          {
            if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
            {
              RoleEnvironment.RequestRecycle();
            }
          }
        };
      });
      return base.OnStart();
    }

    private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
    {
      // If a configuration setting is changing
      if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
      {
        // Set e.Cancel to true to restart this role instance
        e.Cancel = true;
      }
    }
  }
}
