﻿#region Usings
using System;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using Core.Logic;
#endregion

namespace BankWorkerRole1
{
  public class WorkerRole : RoleEntryPoint
  {
    private CloudQueue _accountTransactionLoggQueue;

    public override void Run()
    {
      // This is a sample worker implementation. Replace with your logic.
      Trace.WriteLine("BankWorkerRole1 entry point called", "Information");

      while (true)
      {
        ProcessMessage();

        Thread.Sleep(1000);

        Trace.WriteLine("Working", "Information");
      }
    }

    private void ProcessMessage()
    {
      //Get an message and set the visibility to twenty seconds
      if (_accountTransactionLoggQueue != null)
      {
        CloudQueueMessage msg = _accountTransactionLoggQueue.GetMessage(new TimeSpan(0, 0, 20));
        if (msg != null)
        {
          if (!msg.AsString[0].Equals('<'))
          {
            var str = msg.AsString.Split('|');
            if (str.Length == 4)
            {
              //userid|accountid|transactionid|amount
              UserAccountSqlAzureAction ds = new UserAccountSqlAzureAction(new Core.DataAccess.UserAccountDB("ConnStr"));
              try
              {
                Trace.WriteLine(String.Format("About to insert data to DB:{0}", str), "Information");
        
                ds.UpdateUserAccountBalance(new Guid(str[0]), new Guid(str[1]), double.Parse(str[3]));
                
                Trace.WriteLine(msg.AsString, "Information");
                
                _accountTransactionLoggQueue.DeleteMessage(msg);
                
                Trace.WriteLine(String.Format("deleted:{0}", str), "Information");
              }
              catch (Exception ex)
              {
                Trace.WriteLine(String.Format("fail to insert:{0}", str, ex.Message), "Error");
              }
            }
          }
        }
      }
    }

    public override bool OnStart()
    {
      // Set the maximum number of concurrent connections 
      ServicePointManager.DefaultConnectionLimit = 12;

      // Get the factory configuration so that it can be edited
      DiagnosticMonitorConfiguration config = DiagnosticMonitor.GetDefaultInitialConfiguration();

      // Set scheduled transfer interval for infrastructure logs to 1 minute
      config.DiagnosticInfrastructureLogs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);

      // Specify a logging level to filter records to transfer
      config.DiagnosticInfrastructureLogs.ScheduledTransferLogLevelFilter = LogLevel.Error;

      // Set scheduled transfer interval for user's Windows Azure Logs to 1 minute
      config.Logs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);

      DiagnosticMonitor.Start("DiagnosticsConnectionString", config);

      // For information on handling configuration changes
      // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
      RoleEnvironment.Changing += RoleEnvironmentChanging;

      CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
      {
        configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));
        RoleEnvironment.Changed += (sender, arg) =>
        {
          if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
              .Any((change) => (change.ConfigurationSettingName == configName)))
          {
            if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
            {
              RoleEnvironment.RequestRecycle();
            }
          }
        };
      });

      // initialize queue storage
      InitQueueStorage();

      return base.OnStart();
    }

    private void InitQueueStorage()
    {
      var storageAccount = CloudStorageAccount.FromConfigurationSetting("DataConnectionString");

      CloudQueueClient queueStorage = storageAccount.CreateCloudQueueClient();
      _accountTransactionLoggQueue = queueStorage.GetQueueReference(Core.Helpers.Queues.AccountTransactionLoggQueue);

      Trace.TraceInformation("Creating account transaction logg queue...");

      bool storageInitialized = false;
      while (!storageInitialized)
      {
        try
        {
          // create the message queue
          _accountTransactionLoggQueue.CreateIfNotExist();
          storageInitialized = true;
        }
        catch (StorageClientException e)
        {
          if (e.ErrorCode == StorageErrorCode.TransportError)
          {
            Trace.TraceError("Storage services initialization failure. "
              + "Check your storage account configuration settings. If running locally, "
              + "ensure that the Development Storage service is running. Message: '{0}'", e.Message);
            System.Threading.Thread.Sleep(5000);
          }
          else
          {
            throw;
          }
        }
      }
    }

    private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
    {
      // If a configuration setting is changing
      if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
      {
        // Set e.Cancel to true to restart this role instance
        e.Cancel = true;
      }
    }
  }
}
