﻿using System;
using System.ServiceModel;
using System.ServiceModel.Web;
using Core.Logic;

namespace Core.Services
{
  #region SOAP SERVICES
  [ServiceContract]
  public interface IAccountService : IAccountServiceBase
  {
    [OperationContract]
    void CreateAccount(Core.Models.UserAccount userAccount);

    [OperationContract]
    Core.Models.UserAccount GetUserAccount(Guid userId, Guid accountId);

    [OperationContract]
    Core.Models.UserAccount[] GetUserAccounts(Guid userId);

  }
  [ServiceContract]
  public interface IAccountServiceBase
  {
    IUserAccountAction<Core.Models.UserAccount> Action { get; set; }

    [OperationContract]
    double GetAccountBalance(Guid userId, Guid accountId);

    [OperationContract]
    Core.Models.TransactionResponse InsertMoney(Core.Models.AccountTransactionRequest transRequest);

    [OperationContract]
    Core.Models.TransactionResponse WithdrawMoney(Core.Models.AccountTransactionRequest transRequest);

    [OperationContract]
    Core.Models.TransactionResponse ReliableInsertMoney(Core.Models.AccountTransactionRequest transRequest);

    [OperationContract]
    Core.Models.TransactionResponse ReliableWithDrawMoney(Core.Models.AccountTransactionRequest transRequest);
  }
  #endregion

  #region REST SERVICES
  [ServiceContract]
  public interface IAccountServiceRest
  {
    [OperationContract]
    [WebInvoke(Method="PUT",BodyStyle = WebMessageBodyStyle.Bare,ResponseFormat=WebMessageFormat.Xml)]
    void CreateAccount(Core.Models.UserAccount userAccount);

    [OperationContract]
    [WebGet(UriTemplate="accounts/{userId}/{accountId}")]
    Core.Models.UserAccount GetUserAccount(string userId, string accountId);

    [OperationContract]
    [WebGet(UriTemplate="accounts/{userId}")]
    Core.Models.UserAccount[] GetUserAccounts(string userId);

    [OperationContract]
    [WebGet(UriTemplate = "account/balance/{userId}/{accountId}")]
    double GetAccountBalance(string userId, string accountId);

    [OperationContract]
    [WebInvoke(BodyStyle = WebMessageBodyStyle.WrappedRequest, RequestFormat = WebMessageFormat.Xml, ResponseFormat = WebMessageFormat.Xml)]
    Core.Models.TransactionResponse InsertMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse WithDrawMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse ReliableInsertMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse ReliableWithDrawMoney(Core.Models.AccountTransactionRequest req);

  }
  [ServiceContract]
  public interface IAccountServiceBaseRest
  {
    [OperationContract]
    [WebGet(UriTemplate="account/balance/{userId}/{accountId}")]
    double GetAccountBalance(string userId, string accountId);

    [OperationContract]
    [WebInvoke(BodyStyle = WebMessageBodyStyle.WrappedRequest,RequestFormat=WebMessageFormat.Xml,ResponseFormat=WebMessageFormat.Xml)]
    Core.Models.TransactionResponse InsertMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse WithDrawMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse ReliableInsertMoney(Core.Models.AccountTransactionRequest req);

    [OperationContract]
    [WebInvoke()]
    Core.Models.TransactionResponse ReliableWithDrawMoney(Core.Models.AccountTransactionRequest req);
  }
#endregion

  //[ServiceContract]
  //public interface IAccountService<T>
  //{
  //  [OperationContract]
  //  void CreateAccount(T userAccount);

  //  [OperationContract]
  //  T GetUserAccount(Guid userId, Guid accountId);

  //  [OperationContract]
  //  double GetAccountBalance(Guid userId, Guid accountId);

  //  [OperationContract]
  //  T[] GetUserAccounts(Guid userId);

  //  [OperationContract]
  //  Core.Types.TransactionStatus InsertMoney(Guid userId, Guid accountId, Guid transactionId, double amount);

  //  [OperationContract]
  //  Core.Types.TransactionStatus RedrawMoney(Guid userId, Guid accountId, Guid transactionId, double amount);

  //  [OperationContract]
  //  Core.Types.TransactionStatus ReliableInsertMoney(Guid userId, Guid accountId, Guid transactionId, double amount,bool serializeToQueue);

  //  [OperationContract]
  //  Core.Types.TransactionStatus ReliableRedrawMoney(Guid userId, Guid accountId, Guid transactionId, double amount, bool serializeToQueue);
  //}
  

  
}
