﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.TableStorage;
using System.Data.Services.Client;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;

namespace Core.DataContext
{
  /// <summary>
  /// Accessing Azure Table Storage with ADO.Net data services
  /// DataServiceQuery for Async access
  /// </summary>
  public class UserAccountDataContext : TableServiceContext, IDisposable
  {
    public enum TableNames
    {
      UserAccountTransactionTable,
      UserAccountBalanceTable,
      AccountTransactionLoggTable
    }

    public DataServiceQuery<AccountTransactionLog> AccountTransactionLoggTable
    {
      get
      {
        return CreateQuery<AccountTransactionLog>(TableNames.AccountTransactionLoggTable.ToString());
      }
    }

    [System.ComponentModel.Description("pk:UserId|Accountid,rk=currency")]
    public DataServiceQuery<UserAccountBalance> UserAccountBalanceTable
    {
      get
      {
        return CreateQuery<UserAccountBalance>(TableNames.UserAccountBalanceTable.ToString());
      }
    }
    public DataServiceQuery<UserAccountTransaction> UserAccountTransactionTable
    {
      get
      {
        return CreateQuery<UserAccountTransaction>(TableNames.UserAccountTransactionTable.ToString());
      }
    }
    public UserAccountDataContext(string baseAdress, StorageCredentials creds)
      : base(baseAdress, creds)
    {
      RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(1));
      Init();
    }
    public UserAccountDataContext()
      : base(CloudStorageAccount.FromConfigurationSetting("DataConnectionString").TableEndpoint.AbsoluteUri,
      CloudStorageAccount.FromConfigurationSetting("DataConnectionString").Credentials)
    {
      RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(1));
      Init();
    }
    private static bool _initalized = false;
    static void Init()
    {
      if (!_initalized)
      {
        var storageAccount = CloudStorageAccount.FromConfigurationSetting("DataConnectionString");
        CloudTableClient.CreateTablesFromModel(typeof(UserAccountDataContext),storageAccount.TableEndpoint.AbsoluteUri,storageAccount.Credentials);
        _initalized = true;
      }
    }
    #region IDisposable Members

    public void Dispose()
    {
      //implement later
    }

    #endregion
  }
}
