﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.Helpers;
using Core.Models;
using Core.Types;
using System.Data;
namespace Core.DataAccess
{
  public class UserAccountDB : DBBase, IUserAccountDB
  {
    public UserAccountDB(string connectionString)
      : base(connectionString)
    { }

    public void CreateUserAccount(UserAccount useraccount)
    {
      this.CreateUserAccount(useraccount.UserId, 
                             useraccount.AccountId, 
                             useraccount.UserAccountStatus,
                             useraccount.UserAccountType, 
                             useraccount.Balance, 
                             useraccount.Currency, 
                             useraccount.CreditCardNumber, 
                             useraccount.Description,
                             useraccount.Country, 
                             useraccount.BankIdentifier);
    }
    public void CreateUserAccount(Guid userId, Guid accountId, AccountStatus userAccountStatus,
                                  AccountType userAccountType, double balance, string currency,
                                  string creditCardNumber, string description,
                                  string country, Int32 bankIdentifier)
    {
      //Validate inarguments!!
      try
      {
        using (System.Data.Common.DbCommand cmd = DB.GetStoredProcCommand(StoredProcedures.CREATEUSERACCOUNT))
        {
          DB.AddInParameter(cmd, "@id", DbType.Guid, accountId);
          DB.AddInParameter(cmd, "@userId", DbType.Guid, userId);
          DB.AddInParameter(cmd, "@status", DbType.Int16, (short)userAccountStatus);
          DB.AddInParameter(cmd, "@type", DbType.Int16, (short)userAccountType);
          DB.AddInParameter(cmd, "@balance", DbType.Double, balance);
          DB.AddInParameter(cmd, "@currency", DbType.StringFixedLength, currency);
          DB.AddInParameter(cmd, "@creditCardNr", DbType.StringFixedLength, creditCardNumber);
          DB.AddInParameter(cmd, "@desc", DbType.StringFixedLength, description);
          DB.AddInParameter(cmd, "@country", DbType.StringFixedLength, country);
          DB.AddInParameter(cmd, "@bankId", DbType.Int32, bankIdentifier);
          DB.AddParameter(cmd, "@RETURN_VALUE", DbType.Int32, 
            ParameterDirection.ReturnValue, "", DataRowVersion.Current, null);

          DB.ExecuteNonQuery(cmd);

          int returnValue = (int)DB.GetParameterValue(cmd, "RETURN_VALUE");
          if (returnValue == -1001)
            throw new Exception("-1001");
        }
      }
      catch
      { throw; }
    }
    public UserAccount[] GetUserAccounts(Guid userId)
    {
      try
      {
        List<UserAccount> userAccounts = new List<UserAccount>();
        using (System.Data.Common.DbCommand cmd = 
          DB.GetStoredProcCommand(StoredProcedures.GETUSERACCOUNTS, new object[] { userId }))
        {
          using (IDataReader dr = DB.ExecuteReader(cmd))
          {
            while (dr.Read())
            {
              UserAccount userAccount = new UserAccount()
              {
                AccountId = dr.GetGuid(0),
                UserAccountStatus = (dr.IsDBNull(1) ? AccountStatus.UnKnown : (AccountStatus)dr.GetInt16(1)),
                UserAccountType = (dr.IsDBNull(2) ? AccountType.Secret : (AccountType)dr.GetInt16(2)),
                Balance = dr.GetDouble(3),
                Currency = (dr.IsDBNull(4) ? String.Empty : dr[4].ToString()),
                CreditCardNumber = (dr.IsDBNull(5) ? String.Empty : dr[5].ToString()),
                Description = (dr.IsDBNull(6) ? String.Empty : dr[6].ToString()),
                Country = (dr.IsDBNull(7) ? String.Empty : dr[7].ToString()),
                BankIdentifier = (dr.IsDBNull(8) ? -1 : dr.GetInt32(8)),
                UpdateTag = (byte[])dr.GetValue(9),
                UserId = userId
              };
              userAccounts.Add(userAccount);
            }
          }
        }
        return userAccounts.ToArray();
      }
      catch
      { throw; }
    }
    public UserAccount GetUserAccountByUserId(Guid userId, Guid accountId)
    {
      try
      {
        UserAccount userAccount = null;
        using (System.Data.Common.DbCommand cmd = 
          DB.GetStoredProcCommand(StoredProcedures.GETUSERACCOUNTBYUSERID, new object[] { userId, accountId }))
        {
          using (IDataReader dr = DB.ExecuteReader(cmd))
          {
            if (dr.Read())
            {
              userAccount = new UserAccount()
              {
                AccountId = dr.GetGuid(0),
                UserAccountStatus = (dr.IsDBNull(1) ? AccountStatus.UnKnown : (AccountStatus)dr.GetInt16(1)),
                UserAccountType = (dr.IsDBNull(2) ? AccountType.Secret : (AccountType)dr.GetInt16(2)),
                Balance = dr.GetDouble(3),
                Currency = (dr.IsDBNull(4) ? String.Empty : dr[4].ToString()),
                CreditCardNumber = (dr.IsDBNull(5) ? String.Empty : dr[5].ToString()),
                Description = (dr.IsDBNull(6) ? String.Empty : dr[6].ToString()),
                Country = (dr.IsDBNull(7) ? String.Empty : dr[7].ToString()),
                BankIdentifier = (dr.IsDBNull(8) ? -1 : dr.GetInt32(8)),
                UpdateTag = (byte[])dr.GetValue(9),
                UserId = userId
              };
            }
          }
        }
        return userAccount;
      }
      catch
      { throw; }
    }
    public void UpdateuserAccountBalance(Guid accountId, double balance, byte[] updateTag)
    {
      //Validate user length nulls....
      try
      {
        using (System.Data.Common.DbCommand cmd = DB.GetStoredProcCommand(StoredProcedures.UPDATEUSERACCOUNTBALANCE))
        {
          DB.AddInParameter(cmd, "@id", DbType.Guid, accountId);
          DB.AddInParameter(cmd, "@balance", DbType.Double, balance);
          DB.AddInParameter(cmd, "@updateTag", DbType.Binary, updateTag);
          DB.AddParameter(cmd, "@RETURN_VALUE", DbType.Int32, ParameterDirection.ReturnValue, "", DataRowVersion.Current, null);

          DB.ExecuteNonQuery(cmd);

          int returnValue = (int)DB.GetParameterValue(cmd, "RETURN_VALUE");
          if (returnValue == -2001)//not matching updatetag DB returning error code to handle correctly
            throw new Exception("Another user trying to update");//Use custom exception here
        }
      }
      catch
      { throw; }
    }
  }
}
