﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.Models;
using Core.Types;
using Core.TableStorage;

namespace Core.Extensions
{

  /// <summary>
  /// we have several options when transforming data, from use of xmlserialization to read datacontract,
  /// to saving rows in table storade entity..
  /// </summary>
  public static class DataTransformations
  {

    public static UserAccountAdvanced TransformToUserAccountAdvanced(this UserAccount ucTotransform)
    {
      //TODO:VALIDATE arguments
      if (ucTotransform == null)
        return null;
      UserAccountAdvanced uc = new UserAccountAdvanced();
      uc.Balance = ucTotransform.Balance;
      uc.BankAccountNumber = String.Format("{0}|{1}|{2}|{3}", ucTotransform.Country, ucTotransform.BankIdentifier, ucTotransform.AccountId, ucTotransform.Currency);
      uc.CreditCardNumber = ucTotransform.CreditCardNumber;
      uc.UserAccountStatus = ucTotransform.UserAccountStatus;
      uc.UserAccountType = ucTotransform.UserAccountType;
      uc.UserId = ucTotransform.UserId;

      return uc;
    }
    public static UserAccount TransformToUserAccountBase(this UserAccountAdvanced ucTransformed)
    {
      if (ucTransformed == null)
        return null;
      //TODO:VALIDATE arguments
      //country:bankidentifier:accountid:currency
      string[] accountNumberParts = ucTransformed.BankAccountNumber.Split('|');
      UserAccount uc = new UserAccount();
      Int32 bankId = 0;
      Guid accountId = new Guid(accountNumberParts[2]);
      string currency = accountNumberParts[3];
      string country = accountNumberParts[0];
      if (!Int32.TryParse(accountNumberParts[1], out bankId))
        return uc;

      uc.AccountId = accountId;
      uc.Balance = ucTransformed.Balance;
      uc.BankIdentifier = bankId;
      uc.CreditCardNumber = ucTransformed.CreditCardNumber;
      uc.Country = country;
      uc.Currency = currency;
      uc.UserAccountStatus = ucTransformed.UserAccountStatus;
      uc.UserAccountType = ucTransformed.UserAccountType;
      uc.UserId = ucTransformed.UserId;

      return uc;
    }
    
    public static UserAccountWwi TransformToUserAccountNormal(this UserAccount ucToTransform)
    {
      if (ucToTransform == null)
        return null;
      UserAccountWwi uc = new UserAccountWwi();
      uc.Balance = ucToTransform.Balance;
      uc.BankAccountNumber = string.Format("{0}|{1}", ucToTransform.Currency, ucToTransform.AccountId);
      uc.Country = ucToTransform.Country;
      uc.UserId = ucToTransform.UserId;
      return uc;
    }
    public static UserAccount TransformToUserAccountBase(this UserAccountWwi ucNormal)
    {
      if (ucNormal==null)
        return null;
      //bankaccount number is currency:accountid
      string[] accountNumberParts = ucNormal.BankAccountNumber.Split('|');
      string currency = accountNumberParts[0];
      Guid accountId = new Guid(accountNumberParts[1]);

      UserAccount uc = new UserAccount();
      uc.AccountId = accountId;
      uc.Balance = ucNormal.Balance;
      uc.BankIdentifier = 0;
      uc.Country = ucNormal.Country;
      uc.Currency = currency;
      uc.UserAccountStatus = AccountStatus.UnKnown;
      uc.UserAccountType = AccountType.Regular;
      uc.UserId = ucNormal.UserId;

      return uc;
    }
    
  }
  
  public static class Validators
  {

    public static bool IsParitionkeyValid(this UserAccountTransaction uac,out string[] values)
    {
      string[] sep = uac.PartitionKey.Split('|');
      values = sep;
      return (sep.Count() != 2);
    }
  }
  public static class ClosureExtensions
  {
    public static IEnumerable<TResult> Map<T, TResult>(this IEnumerable<T> items, Func<T, TResult> func)
    {
      foreach (var item in items)
        yield return func(item);
    }
    public static void ForEach<T>(this IEnumerable<T> items, Action<T> action)
    {
      if (items == null) throw new ArgumentNullException("items");
      if (action == null) throw new ArgumentNullException("action");

      foreach (var item in items)
        action(item);
    }
  }

}
