﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Xml;
using System.Runtime.Serialization;
using System.IO;
using Core.Extensions;
using System.ServiceModel;
namespace Core.Helpers
{
  public static class StoredProcedures
  {
    internal const string GETUSERBYID = "dbo.GetUserById";
    internal const string CREATENEWUSER = "dbo.CreateNewUser";
    internal const string GETUSERACCOUNTS = "dbo.GetUserAccounts";
    internal const string CREATEUSERACCOUNT = "dbo.CreateUserAccount";
    internal const string GETUSERACCOUNTBYUSERID = "dbo.GetUserAccountByUserId";
    internal const string UPDATEUSERACCOUNTBALANCE = "dbo.UpdateUserAccountBalance";

  }
  public static class Queues
  {
    public const string AccountTransactionsQueue = "accounttransactions";
    public const string AccountTransactionLoggQueue = "accounttransactionslogg";
  }
  
  public static class WCFUsing
  {
    public static void Using<ServiceT>(Action<ServiceT> action) where ServiceT : ICommunicationObject, new()
    {
      var service = new ServiceT();
      bool success = false;
      try
      {
        action(service);
        service.Close();
        success = true;
      }
      finally
      {
        if (!success)
        {
          service.Abort();
        }
      }
    }
    public static void Using<TClient>(TClient client, Action<TClient> action) where TClient : ICommunicationObject
    {
      if (client == null)
      {
        throw new ArgumentNullException("client");
      }
      if (action == null)
      {
        throw new ArgumentNullException("action");
      }

      try
      {
        client.Open();
        action(client);
        client.Close();
      }
      finally
      {
        if (client.State != CommunicationState.Closed)
        {
          client.Abort();
        }
      }
    }
  }



}
