﻿using System;
using System.Collections.Generic;
using System.Linq;
using Core.Models;
using Core.Types;

namespace Core.Logic
{
  public class UserAccountAction : Core.Logic.IUserAccountAction<Core.Models.UserAccount>
  {
    public IUserAccountStorageAction StorageAction { get; set; }
    public IUserAccountSqlAzureAction SqlAzureAction { get; set; }

    public UserAccountAction(IUserAccountStorageAction accountStorageAction,IUserAccountSqlAzureAction sqlAzureAction)
    {
      this.SqlAzureAction = sqlAzureAction;
      this.StorageAction = accountStorageAction;
    }
    public void CreateUserAccount(Core.Models.UserAccount userAccount)
    {
      if(userAccount!=null)
      {
        try
        {
          StorageAction.CreateUserAccount(
            new Core.TableStorage.UserAccountBalance(userAccount.UserId, 
                                                      userAccount.AccountId) 
                                                      { Balance = userAccount.Balance });
          SqlAzureAction.CreateUserAccount(userAccount);
        }
        catch(Exception ex)
        {
          StorageAction.DeleteUserAccount(userAccount.UserId, userAccount.AccountId);
          throw ex;
        }
        //TRANSACTION HERE, AT FAIL UNDO ACTION
        
      }
    }
    public List<Core.Models.UserAccount> GetUserAccounts(Guid userId)
    {
      if (userId == Guid.Empty)
      {
        //throw new ServiceException("user id invalid");
        throw new Exception("user id invalid");
      }
      try
      {
        return SqlAzureAction.GetUserAccounts(userId).ToList();
      }
      catch
      {throw;}
    }
    public double GetAccountBalance(Guid userId,Guid accountId)
    {
      if (userId == Guid.Empty || accountId==Guid.Empty)
      {
        //throw new ServiceException("user id invalid");
        throw new Exception("user id invalid");
      }
      try
      {
        return StorageAction.GetAccountBalance(userId, accountId);
      }
      catch
      {throw;}
    }
    public Core.Models.UserAccount GetUserAccount(Guid userId, Guid accountId)
    {
      if (userId == Guid.Empty|| accountId==Guid.Empty)
      {
        //throw new ServiceException("user id invalid");
        throw new Exception("user id invalid");
      }
      try
      {
        return SqlAzureAction.GetUserAccountByUserId(userId, accountId);
      }
      catch
      { throw; }
    }

    public Core.Models.TransactionResponse InsertMoney(Guid userId, Guid accountId, Guid transactionId, double amount)
    {
      Core.Models.TransactionResponse status = new TransactionResponse();
      if (!(userId == Guid.Empty || accountId == Guid.Empty || transactionId == Guid.Empty || amount<=0))
      {
        //throw new ServicesException...
        
        try
        {
          Core.TableStorage.UserAccountTransaction uct = 
            new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = amount };
          status = StorageAction.InsertOrRedrawMoneyWithTableStorage(uct);
          
          SqlAzureAction.UpdateUserAccountBalance(userId, accountId, amount);
        }
        catch 
        { 
          status.Message= "Error while processing";
          status.TransactionStatus = TransactionStatus.Failed;
          throw; 
        }
      }

      return status;
    }

    public Core.Models.TransactionResponse WithDrawMoney(Guid userId, Guid accountId, Guid transactionId, double amount)
    {
      Core.Models.TransactionResponse status = new TransactionResponse();
      if (!(userId == Guid.Empty || accountId == Guid.Empty || transactionId == Guid.Empty || amount <= 0))
      {
        //throw new ServicesException...
        // [Description("PartitionKey=UserId|AccountId, Rowkey=TransactionId")]
        try
        {
          Core.TableStorage.UserAccountTransaction uct = 
            new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = -amount };
          status = StorageAction.InsertOrRedrawMoneyWithTableStorage(uct);

          SqlAzureAction.UpdateUserAccountBalance(userId, accountId, amount);
        }
        catch
        { 
          status.Message= "Error while processing";
          status.TransactionStatus = TransactionStatus.Failed;
          throw; 
        }
      }
      return status;
    }

    public Core.Models.TransactionResponse ReliableInsertMoney(Guid userId, Guid accountId, Guid transactionId, double amount, bool serializeToQueue)
    {
      try
      {
        Core.Models.TransactionResponse response = new TransactionResponse();
        if (serializeToQueue)
          response = StorageAction.ReliableHandleMoneyInQueueAsXml(
            new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = amount });
        else
          response = StorageAction.ReliableHandleMoneyInQueue(
              new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = amount });

        return response;
      }
      catch { throw; }
    }

    public Core.Models.TransactionResponse ReliableWithDrawMoney(Guid userId, Guid accountId, Guid transactionId, double amount, bool serializeToQueue)
    {
      Core.Models.TransactionResponse response = new TransactionResponse();
      try
      {
        if (serializeToQueue)
          response = StorageAction.ReliableHandleMoneyInQueueAsXml(
              new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = -amount });
        else
          response = StorageAction.ReliableHandleMoneyInQueue(
              new Core.TableStorage.UserAccountTransaction(userId, accountId, transactionId) { Amount = -amount });
        return response;
      }
      catch { throw; }

    }

  }
}
