﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.Logic;
using System.ServiceModel;
using System.ServiceModel.Web;
using Core.Models;

namespace Core.Services
{
  #region SOAP Service implementation
  [ServiceBehavior(AddressFilterMode = AddressFilterMode.Any)]
  public class UserAccountService : IAccountService
  {
    public IUserAccountAction<Core.Models.UserAccount> Action
    {get;set;}

    public UserAccountService()
    {
      
      //REPLACE LATER WITH INJECTION IN CONFIG!!!!!!!!!!, replace hardcoded strings
      Action = new UserAccountAction(new UserAccountStorageAction(), new UserAccountSqlAzureAction(new Core.DataAccess.UserAccountDB("ConnStr")));
    }
    #region IAccountService<T> Members

    public void CreateAccount(Core.Models.UserAccount userAccount)
    {
      if (userAccount != null)
      {
        //TRANSFORM HERE TO CORRECT CONTRACT
        Action.CreateUserAccount(userAccount);
      }
    }

    public UserAccount GetUserAccount(Guid userId, Guid accountId)
    {
      return Action.GetUserAccount(userId, accountId);
    }

    public double GetAccountBalance(Guid userId, Guid accountId)
    {
      return Action.GetAccountBalance(userId, accountId);
    }

    public UserAccount[] GetUserAccounts(Guid userId)
    {
      return Action.GetUserAccounts(userId).ToArray();
    }

    public TransactionResponse InsertMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
      {
        return Action.InsertMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount);
      }
      else
        return new TransactionResponse { 
          TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };

    }

    public TransactionResponse WithdrawMoney(AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.WithDrawMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount);
      else
        return new TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };
    }


    public Core.Models.TransactionResponse ReliableInsertMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.ReliableInsertMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount, true);
      else 
        return new TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };      
    }

    public TransactionResponse ReliableWithDrawMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.ReliableWithDrawMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount, false);
      else
        return new TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };            
    }

    #endregion
  
  }
  #endregion

  #region REST Service implementation

  public class UserAccountServiceRest : IAccountServiceRest 
  {
    public IUserAccountAction<Core.Models.UserAccount> Action
    { get; set; }

    public UserAccountServiceRest()
    {
      //REPLACE LATER WITH INJECTION IN CONFIG!!!!!!!!!!, replace hardcoded strings
      Action = new UserAccountAction(new UserAccountStorageAction(), new UserAccountSqlAzureAction(new Core.DataAccess.UserAccountDB("ConnStr")));
    }

    #region IAccountService<T> Members

    public void CreateAccount(Core.Models.UserAccount userAccount)
    {
      if (userAccount != null)
      {
        //TRANSFORM HERE TO CORRECT CONTRACT
        Action.CreateUserAccount(userAccount);
      }
    }

    #endregion

    #region IAccountServiceRest Members
    
    public TransactionResponse ReliableInsertMoney(AccountTransactionRequest req)
    {
      IncomingWebRequestContext inRequest = WebOperationContext.Current.IncomingRequest;
      Guid messageId = this.GetETag(inRequest.Headers[System.Net.HttpRequestHeader.IfMatch]);
      return Action.ReliableInsertMoney(req.UserId, req.AccountId, messageId, req.Amount, true);
    }

    public UserAccount GetUserAccount(string userId, string accountId)
    {
      return Action.GetUserAccount(new Guid(userId), new Guid(accountId));
    }

    public UserAccount[] GetUserAccounts(string userId)
    {
      return Action.GetUserAccounts(new Guid(userId)).ToArray();
    }

    public double GetAccountBalance(string userId, string accountId)
    {
      return Action.GetAccountBalance(new Guid(userId), new Guid(accountId));
    }

    public TransactionResponse InsertMoney(AccountTransactionRequest req)
    {
      return Action.InsertMoney(req.UserId, req.AccountId, Guid.NewGuid(), req.Amount);
    }

    public TransactionResponse WithDrawMoney(AccountTransactionRequest req)
    {
      return Action.WithDrawMoney(req.UserId, req.AccountId, Guid.NewGuid(), req.Amount);
    }

    

    private Guid GetETag(string p)
    {
      //if (p.IsNullOrEmpty(incomingEtag))
      //      {
      //          throw new WebProtocolException(HttpStatusCode.BadRequest, "The If-Match header was not specified for the request", null);
      //      }
      //      if (!string.Equals(incomingEtag, expectedEtag, StringComparison.Ordinal))
      //      {
      //          throw new WebProtocolException(HttpStatusCode.Conflict, String.Format("The resource has an Etag different from '{0}'. Please get the latest copy of the resource.", incomingEtag), null);
      //      }
       return new Guid(p.Replace("\"","").ToString());
    }

    public TransactionResponse ReliableWithDrawMoney(AccountTransactionRequest req)
    {
      return Action.ReliableWithDrawMoney(req.UserId, req.AccountId, Guid.NewGuid(), req.Amount, false);
    }

    #endregion
  }
#endregion
}
