﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.Logic;
using Core.Extensions;
using System.ServiceModel;

namespace Core.Services
{
  [ServiceBehavior(AddressFilterMode = AddressFilterMode.Any)]
  public class UserAccountServiceNormal:IAccountServiceNormal//IAccountService<Core.Models.UserAccountNormal>
  {
    #region IAccountService Members
    public IUserAccountAction<Core.Models.UserAccount> Action { get; set; }
    public UserAccountServiceNormal()
    {
      //REPLACE LATER WITH INJECTION IN CONFIG!!!!!!!!!!
      Action = new UserAccountAction(new UserAccountStorageAction(), new UserAccountSqlAzureAction(new Core.DataAccess.UserAccountDB("ConnStr")));
    }
    public void CreateAccount(Core.Models.UserAccountWwi userAccount)
    {
      //VALIDATION CODE HERE....
      if (userAccount != null)
      {
        //TRANSFORM HERE TO CORRECT CONTRACT
        Action.CreateUserAccount(userAccount.TransformToUserAccountBase());
      }
    }

    public Core.Models.UserAccountWwi GetUserAccount(Guid userId, Guid accountId)
    {
      //if(userId|| accountId==Guid.Empty)
        //throw new ServiceException(Fault)
      return Action.GetUserAccount(userId, accountId).TransformToUserAccountNormal();
    }

    public double GetAccountBalance(Guid userId, Guid accountId)
    {
      return Action.GetAccountBalance(userId, accountId);
    }

    public Core.Models.UserAccountWwi[] GetUserAccounts(Guid userId)
    {
      return Action.GetUserAccounts(userId).Map(a =>  a.TransformToUserAccountNormal()).ToArray();
    }

    public Core.Models.TransactionResponse InsertMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.InsertMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount);
      else
        return new Core.Models.TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };      
    }

    public Core.Models.TransactionResponse WithdrawMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.WithDrawMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount);
      else
        return new Core.Models.TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };      

    }

    #endregion



    #region IAccountService<UserAccountNormal> Members


    public Core.Models.TransactionResponse ReliableInsertMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.ReliableInsertMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount, false);
      else
        return new Core.Models.TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };      
    }

    public Core.Models.TransactionResponse ReliableWithDrawMoney(Core.Models.AccountTransactionRequest transRequest)
    {
      var messageId = OperationContext.Current.IncomingMessageHeaders.MessageId;
      Guid messageGuid = Guid.Empty;
      if (messageId.TryGetGuid(out messageGuid))
        return Action.ReliableWithDrawMoney(transRequest.UserId, transRequest.AccountId, messageGuid, transRequest.Amount, true);
      else
        return new Core.Models.TransactionResponse { TransactionStatus = Core.Types.TransactionStatus.Failed, Message = "MessageId invalid" };      
    }

    #endregion
  }
}
