﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ServiceModel.Web;
using Microsoft.Http;
using Microsoft.ServiceModel.Web.SpecializedServices;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.Xml;
using System.Net;
using Core.Models;



namespace RestClient
{
  class Program
  {
    static void Main(string[] args)
    {

      Guid userId = Guid.NewGuid();
      Guid accountId = Guid.NewGuid();
      Guid transactionId = Guid.NewGuid();
      string BankRestServiceUrl = "http://127.0.0.1:8080/BankAccountServiceRest.svc/";
      string UserRestServiceUrl = "http://127.0.0.1:8080/UserBankServiceRest.svc/";

      Core.Models.User user = new Core.Models.User()
      {
        UserId = userId,
        Address = "MyAdressRest",
        FirstName = "RestMan",
        LastName = "RestLast",
        PhoneNumber = "0332133333",
        SocSecNumber = "730129"
      };
      Core.Models.UserAccount uc = new Core.Models.UserAccount
      {
        UserId = userId,
        AccountId = accountId,
        UserAccountStatus = Core.Types.AccountStatus.Online,
        UserAccountType = Core.Types.AccountType.Regular,
        Balance = 1000,
        Currency = "SEK",
        Description = "desc",
        CreditCardNumber = "323333",
        Country = "SWE",
        BankIdentifier = 46
      };

      //Create User
      using (HttpClient httpClient = new HttpClient(UserRestServiceUrl))
      {
        var usercontent = HttpContentExtensions.CreateDataContract(user, Encoding.Default, "text/xml", null);
        var responsefromCreateUser = httpClient.Put("CreateUser", usercontent);
        if (responsefromCreateUser.StatusCode != HttpStatusCode.OK)
        {
          throw new Exception("something is not right");
        }
        var response = httpClient.Get(String.Format("{0}/users/{1}", UserRestServiceUrl, userId));
        var userAccount = response.EnsureStatusIsSuccessful().Content.ReadAsDataContract<Core.Models.User>();
      }



      //Create Account
      using (HttpClient httpClient = new HttpClient(BankRestServiceUrl))
      {
        var accountcontent = HttpContentExtensions.CreateDataContract(uc, Encoding.Default, "text/xml", null);
        var responseaccount = httpClient.Put("CreateAccount", accountcontent);
        if (responseaccount.StatusCode != HttpStatusCode.OK)
        {
          throw new Exception("something is not right");
        }

        AccountTransactionRequest actRequest = 
          new AccountTransactionRequest { AccountId = accountId, UserId = userId, Amount = 1000 };

        httpClient.DefaultHeaders.IfMatch = new Microsoft.Http.Headers.EntityTag(transactionId.ToString());

        HttpContent content = HttpContentExtensions.CreateDataContract(actRequest, Encoding.Default, "text/xml", null);
        var response = httpClient.Post("ReliableInsertMoney", content);
        
        if (response.StatusCode != HttpStatusCode.OK)
        {
          throw new Exception("Unable to Insert Money");
        }


        var transactiontresponse = 
          response.EnsureStatusIsSuccessful().Content.ReadAsDataContract<Core.Models.TransactionResponse>();
      }


      // same ETAG
      using (HttpClient httpClient = new HttpClient(BankRestServiceUrl))
      {
        AccountTransactionRequest actRequestFailed =
          new AccountTransactionRequest { AccountId = accountId, UserId = userId, Amount = 222 };
        httpClient.DefaultHeaders.IfMatch = new Microsoft.Http.Headers.EntityTag(transactionId.ToString());
        HttpContent contentFailed = HttpContentExtensions.CreateDataContract(actRequestFailed, Encoding.Default, "text/xml", null);
        var responseFailed = httpClient.Post("ReliableInsertMoney", contentFailed);
        if (responseFailed.StatusCode != HttpStatusCode.OK)
        {
          throw new Exception("Unable to Insert Money");
        }
        var transactiontresponsefailed =
          responseFailed.EnsureStatusIsSuccessful().Content.ReadAsDataContract<Core.Models.TransactionResponse>();

      }

      //Get User
      using (HttpClient httpClient = new HttpClient(UserRestServiceUrl))
      {
        var response = httpClient.Get(String.Format("{0}/users/{1}", UserRestServiceUrl,userId));
        var userAccount = response.EnsureStatusIsSuccessful().Content.ReadAsDataContract<Core.Models.User>();
      }

      //Get Account
      using (HttpClient httpClient = new HttpClient(BankRestServiceUrl))
      {
        var response = httpClient.Get(String.Format("{0}/accounts/{1}/{2}", BankRestServiceUrl, userId, accountId));
        var userAccount = response.EnsureStatusIsSuccessful().Content.ReadAsDataContract<Core.Models.UserAccount>();
      }
      

    }
  }
}
