﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using TestServices.BankAccountServiceCustomerB;
namespace TestServices
{
  class Program
  {
    static void Main(string[] args)
    {
      System.Xml.UniqueId messageId = new System.Xml.UniqueId(Guid.NewGuid());
      double balance = 0;
      var userId = Guid.NewGuid();
      var accountId = Guid.NewGuid();
      Guid transactionId = Guid.NewGuid();
      BankAccountServiceCustomerB.AccountServiceNormalClient svc1 = new TestServices.BankAccountServiceCustomerB.AccountServiceNormalClient();


      //CREATE USER
      WCFUsing.Using(new UserBankService.UserBankServiceClient(),
      client =>
      {
        using (new OperationContextScope(client.InnerChannel))
        {
          OperationContext.Current.OutgoingMessageHeaders.MessageId = messageId;
          client.CreateUser(new UserBankService.User()
          {
            UserId = userId,
            Address = "My Adress",
            FirstName = "Miralem",
            LastName = "Kruskic",
            PhoneNumber = "02133333",
            SocSecNumber = "2323232"
          });
        }
      });

      //CREATE ACCOUNT AND INSERT 100 SEK
      WCFUsing.Using(new BankAccountServiceCustomerB.AccountServiceNormalClient(),
      client =>
      {
        using (new OperationContextScope(client.InnerChannel))
        {
          OperationContext.Current.OutgoingMessageHeaders.MessageId = new System.Xml.UniqueId(Guid.NewGuid());
          client.CreateAccount(new BankAccountServiceCustomerB.UserAccountWwi { Balance = 100.00, BankAccountNumber = String.Format("{0}|{1}", "SEK", accountId), Country = "SWE", UserId = userId });
        }
      });

      
      WCFUsing.Using(new BankAccountServiceCustomerB.AccountServiceNormalClient(),
      client =>
      {
        using (new OperationContextScope(client.InnerChannel))
        {
          OperationContext.Current.OutgoingMessageHeaders.MessageId = new System.Xml.UniqueId(Guid.NewGuid());
          var q1 = client.ReliableWithDrawMoney(new AccountTransactionRequest { UserId = userId, AccountId = accountId, Amount = 33.00 });

          OperationContext.Current.OutgoingMessageHeaders.MessageId = new System.Xml.UniqueId(Guid.NewGuid());
          q1 = client.ReliableInsertMoney(new AccountTransactionRequest { UserId = userId, AccountId = accountId, Amount = 99.00 });

          System.Threading.Thread.Sleep(5000);//wait for worker roles to complete!
          
          double bal = balance = svc1.GetAccountBalance(userId, accountId); //should be 166
          var account1 = svc1.GetUserAccount(userId, accountId);
          var q = svc1.GetUserAccounts(userId);
        }
      });


      //TEST INSERT WITH SAME TRANSACTION ID
      WCFUsing.Using(new BankAccountServiceCustomerB.AccountServiceNormalClient(),
      client =>
      {
        using (new OperationContextScope(client.InnerChannel))
        {
          OperationContext.Current.OutgoingMessageHeaders.MessageId = messageId;
          var q1 = client.ReliableInsertMoney(new AccountTransactionRequest { UserId = userId, AccountId = accountId, Amount = 1000.00 });
          System.Threading.Thread.Sleep(10000);//wait for worker roles to complete!
          var q0 = client.ReliableInsertMoney(new AccountTransactionRequest { UserId = userId, AccountId = accountId, Amount = 1000.00 });
          System.Threading.Thread.Sleep(10000);//wait for worker roles to complete!
          double bal = balance = svc1.GetAccountBalance(userId, accountId); //should be 1166
          //only one insert should be walid because of the messageid in header!
        }
      });

      WCFUsing.Using(new BankAccountServiceCustomerB.AccountServiceNormalClient(),
      client =>
      {
        using (new OperationContextScope(client.InnerChannel))
        {
          OperationContext.Current.OutgoingMessageHeaders.MessageId = new System.Xml.UniqueId(Guid.NewGuid());
          var q2 = client.ReliableWithDrawMoney(new AccountTransactionRequest { UserId = userId, AccountId = accountId, Amount = 300.00 });
          System.Threading.Thread.Sleep(5000);//wait for worker roles to complete!
          double bal = balance = svc1.GetAccountBalance(userId, accountId);//should be 866!
        }
      });
      //result shoud be 866 in DB also!
    }
  }
  public static class WCFUsing
  {
    public static void Using<ServiceT>(Action<ServiceT> action) where ServiceT : ICommunicationObject, new()
    {
      var service = new ServiceT();
      bool success = false;
      try
      {
        action(service);
        service.Close();
        success = true;
      }
      finally
      {
        if (!success)
        {
          service.Abort();
        }
      }
    }
    public static void Using<TClient>(TClient client, Action<TClient> action) where TClient : ICommunicationObject
    {
      if (client == null)
      {
        throw new ArgumentNullException("client");
      }
      if (action == null)
      {
        throw new ArgumentNullException("action");
      }

      try
      {
        client.Open();
        action(client);
        client.Close();
      }
      finally
      {
        if (client.State != CommunicationState.Closed)
        {
          client.Abort();
        }
      }
    }
  }
}


//Code to get those values:

//if (WebOperationContext.IncomingRequest.Headers[HttpRequestHeader.IfNoneMatch] != null)

//{

//  // Do something with etag...

//}

//else if (WebOperationContext.IncomingRequest.Headers[HttpRequestHeader.IfModifiedSince] != null)

//{

//  var date = DateTime.Parse(context.IncomingRequest.Headers[HttpRequestHeader.IfModifiedSince]);

//} 



//Code to set those values:



//WebOperationContext.OutgoingResponse.LastModified = // sets the last modified;

//WebOperationContext.OutgoingResponse.ETag = // sets the Etag

//Code to return an Http 304 status code:

//WebOperationContext.OutgoingResponse.StatusCode = HttpStatusCode.NotModified;

//WebOperationContext.OutgoingResponse.SuppressEntityBody = true;
